@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2013 Kuan-Chung Chiu <buganini@gmail.com>
+ * Copyright (c) 2009-2014 Kuan-Chung Chiu <buganini@gmail.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -31,7 +31,6 @@ typedef struct {
 } Bsdconv;
 
 typedef struct {
-
 	PyObject_HEAD
 	FILE *fp;
 } Bsdconv_file;
@@ -323,7 +322,7 @@ py_bsdconv_conv_file(PyObject *self, PyObject *args)
 #ifndef WIN32
 	struct stat stat;
 	fstat(fileno(inf), &stat);
-	(void) fchown(fileno(otf), stat.st_uid, stat.st_gid);
+	fchown(fileno(otf), stat.st_uid, stat.st_gid);
 	fchmod(fileno(otf), stat.st_mode);
 #endif
 
@@ -518,50 +517,6 @@ py_bsdconv_valid(PyObject *self, PyObject *args)
 		return 0;
 }
 
-static PyMethodDef Bsdconv_methods[] = {
-	{"init",	py_bsdconv_init,	METH_VARARGS,
-		PyDoc_STR("init() -> Initialize/Reset bsdconv instance")},
-	{"ctl",	py_bsdconv_ctl,	METH_VARARGS,
-		PyDoc_STR("ctl(arg_ptr_obj, arg_int) -> Initialize/Reset bsdconv instance")},
-	{"conv",	py_bsdconv_conv,	METH_VARARGS,
-		PyDoc_STR("conv(s) -> Perform conversion")},
-	{"conv_chunk",	py_bsdconv_conv_chunk,	METH_VARARGS,
-		PyDoc_STR("conv_chunk(s) -> Perform conversion without initializing and flushing")},
-	{"conv_chunk_last",	py_bsdconv_conv_chunk_last,	METH_VARARGS,
-		PyDoc_STR("conv_chunk_last(s) -> Perform conversion without initializing")},
-	{"conv_file",	py_bsdconv_conv_file,	METH_VARARGS,
-		PyDoc_STR("conv_file(from_file, to_file) -> Perform conversion with given filename")},
-	{"testconv",	py_bsdconv_testconv,	METH_VARARGS,
-		PyDoc_STR("testconv(s) -> Perform test conversion")},
-	{"testconv_chunk",	py_bsdconv_testconv_chunk,	METH_VARARGS,
-		PyDoc_STR("testconv_chunk(s) -> Perform test conversion without initializing and flushing")},
-	{"testconv_chunk_last",	py_bsdconv_testconv_chunk_last,	METH_VARARGS,
-		PyDoc_STR("testconv_chunk_last(s) -> Perform test conversion without initializing")},
-	{"testconv_file",	py_bsdconv_testconv_file,	METH_VARARGS,
-		PyDoc_STR("testconv_file(from_file) -> Perform test conversion with given filename")},
-	{"counter",	py_bsdconv_counter,	METH_VARARGS,
-		PyDoc_STR("counter([name]) -> Return conversion info")},
-	{"counter_reset",	py_bsdconv_counter_reset,	METH_VARARGS,
-		PyDoc_STR("counter_reset([name]) -> Reset counter, if no name supplied, all counters will be reset")},
-	{NULL,		NULL}		/* sentinel */
-};
-
-static PyNumberMethods Bsdconv_number_methods = {
-#if PY_MAJOR_VERSION < 3
-	.nb_nonzero = (inquiry) py_bsdconv_valid
-#else
-	.nb_bool = (inquiry) py_bsdconv_valid
-#endif
-};
-
-#if PY_MAJOR_VERSION < 3
-static PyObject *
-py_bsdconv_getattr(PyObject *self, char *attrname)
-{
-	return Py_FindMethod(Bsdconv_methods, self, attrname);
-}
-#endif
-
 static PyObject *
 py_bsdconv_repr(PyObject *self, char *attrname)
 {
@@ -584,6 +539,108 @@ py_bsdconv_repr(PyObject *self, char *attrname)
 	return r;
 }
 
+static PyObject *
+py_bsdconv_modules_list(PyObject *self, PyObject *args)
+{
+	PyObject *ret=PyList_New(0);
+	char **list;
+	char **p;
+	int phase_type;
+	if (!PyArg_ParseTuple(args, "i", &phase_type))
+		return NULL;
+	list=bsdconv_modules_list(phase_type);
+	p=list;
+	while(*p!=NULL){
+		PyList_Append(ret, Py_BuildValue("s", *p));
+		bsdconv_free(*p);
+		p+=1;
+	}
+	bsdconv_free(list);
+	return ret;
+}
+
+static PyObject *
+py_bsdconv_module_check(PyObject *self, PyObject *args)
+{
+	PyObject *r;
+	char *s;
+	int type;
+	if (!PyArg_ParseTuple(args, "iz", &type, &s))
+		return NULL;
+	if(bsdconv_module_check(type, s))
+		r=Py_True;
+	else
+		r=Py_False;
+	Py_INCREF(r);
+	return r;
+}
+
+static PyMethodDef Bsdconv_methods[] = {
+	{"init",	py_bsdconv_init,	METH_VARARGS,
+		PyDoc_STR("init() -> Initialize/Reset bsdconv instance")},
+	{"ctl",	py_bsdconv_ctl,	METH_VARARGS,
+		PyDoc_STR("ctl(arg_ptr_obj, arg_int) -> Manipulate the underlying codec parameters")},
+	{"conv",	py_bsdconv_conv,	METH_VARARGS,
+		PyDoc_STR("conv(s) -> Perform conversion")},
+	{"conv_chunk",	py_bsdconv_conv_chunk,	METH_VARARGS,
+		PyDoc_STR("conv_chunk(s) -> Perform conversion without initializing and flushing")},
+	{"conv_chunk_last",	py_bsdconv_conv_chunk_last,	METH_VARARGS,
+		PyDoc_STR("conv_chunk_last(s) -> Perform conversion without initializing")},
+	{"conv_file",	py_bsdconv_conv_file,	METH_VARARGS,
+		PyDoc_STR("conv_file(from_file, to_file) -> Perform conversion with given filename")},
+	{"testconv",	py_bsdconv_testconv,	METH_VARARGS,
+		PyDoc_STR("testconv(s) -> Perform test conversion")},
+	{"testconv_chunk",	py_bsdconv_testconv_chunk,	METH_VARARGS,
+		PyDoc_STR("testconv_chunk(s) -> Perform test conversion without initializing and flushing")},
+	{"testconv_chunk_last",	py_bsdconv_testconv_chunk_last,	METH_VARARGS,
+		PyDoc_STR("testconv_chunk_last(s) -> Perform test conversion without initializing")},
+	{"testconv_file",	py_bsdconv_testconv_file,	METH_VARARGS,
+		PyDoc_STR("testconv_file(from_file) -> Perform test conversion with given filename")},
+	{"counter",	py_bsdconv_counter,	METH_VARARGS,
+		PyDoc_STR("counter([name]) -> Return counter or counters if not specified")},
+	{"counter_reset",	py_bsdconv_counter_reset,	METH_VARARGS,
+		PyDoc_STR("counter_reset([name]) -> Reset counter, if no name supplied, all counters will be reset")},
+	{"insert_phase",	py_bsdconv_insert_phase,	METH_VARARGS | METH_STATIC,
+		PyDoc_STR("insert_phase(conversion, codecs, phase_type, phasen) -> Insert conversion phase into bsdconv conversion string")},
+	{"insert_codec",	py_bsdconv_insert_codec,	METH_VARARGS | METH_STATIC,
+		PyDoc_STR("insert_codec(conversion, codec, phasen, codecn) -> Insert conversion codec into bsdconv conversion string")},
+	{"replace_phase",	py_bsdconv_replace_phase,	METH_VARARGS | METH_STATIC,
+		PyDoc_STR("replace_phase(conversion, codecs, phase_type, phasen) -> Replace conversion phase in the bsdconv conversion string")},
+	{"replace_codec",	py_bsdconv_replace_codec,	METH_VARARGS | METH_STATIC,
+		PyDoc_STR("replace_codec(conversion, codec, phasen, codecn) -> Replace conversion codec in the bsdconv conversion string")},
+	{"error",	py_bsdconv_error,	METH_VARARGS | METH_STATIC,
+		PyDoc_STR("error() -> Return error message")},
+	{"mktemp",	py_bsdconv_mktemp,	METH_VARARGS | METH_STATIC,
+		PyDoc_STR("mktemp() -> Make temporary file")},
+	{"fopen",	py_bsdconv_fopen,	METH_VARARGS | METH_STATIC,
+		PyDoc_STR("fopen() -> Open file")},
+	{"modules_list",	py_bsdconv_modules_list,	METH_VARARGS | METH_STATIC,
+		PyDoc_STR("modules_list() -> list codecs")},
+	{"codecs_list",	py_bsdconv_modules_list,	METH_VARARGS | METH_STATIC,
+		PyDoc_STR("codecs_list() -> list codecs\nDEPRECATED: Use modules_list() instead")},
+	{"module_check",	py_bsdconv_module_check,	METH_VARARGS | METH_STATIC,
+		PyDoc_STR("module_check(type, codec) -> check if a codec is available")},
+	{"codec_check",	py_bsdconv_module_check,	METH_VARARGS | METH_STATIC,
+		PyDoc_STR("codec_check(type, codec) -> check if a codec is available\nDEPRECATED: Use module_check() instead")},
+	{NULL,		NULL}		/* sentinel */
+};
+
+#if PY_MAJOR_VERSION < 3
+static PyObject *
+py_bsdconv_getattr(PyObject *self, char *attrname)
+{
+	return Py_FindMethod(Bsdconv_methods, self, attrname);
+}
+#endif
+
+static PyNumberMethods Bsdconv_number_methods = {
+#if PY_MAJOR_VERSION < 3
+	.nb_nonzero = (inquiry) py_bsdconv_valid
+#else
+	.nb_bool = (inquiry) py_bsdconv_valid
+#endif
+};
+
 static PyTypeObject Bsdconv_Type = {
 #if PY_MAJOR_VERSION < 3
 	PyObject_HEAD_INIT(NULL)
@@ -624,61 +681,8 @@ py_bsdconv_new(PyObject *self, PyObject *args)
 	return ret;
 }
 
-static PyObject *
-py_bsdconv_codecs_list(PyObject *self, PyObject *args)
-{
-	PyObject *ret=PyList_New(0);
-	char **list;
-	char **p;
-	int phase_type;
-	if (!PyArg_ParseTuple(args, "i", &phase_type))
-		return NULL;
-	list=bsdconv_codecs_list(phase_type);
-	p=list;
-	while(*p!=NULL){
-		PyList_Append(ret, Py_BuildValue("s", *p));
-		bsdconv_free(*p);
-		p+=1;
-	}
-	bsdconv_free(list);
-	return ret;
-}
-
-static PyObject *
-py_bsdconv_codec_check(PyObject *self, PyObject *args)
-{
-	PyObject *r;
-	char *s;
-	int type;
-	if (!PyArg_ParseTuple(args, "iz", &type, &s))
-		return NULL;
-	if(bsdconv_codec_check(type, s))
-		r=Py_True;
-	else
-		r=Py_False;
-	Py_INCREF(r);
-	return r;
-}
 
 static PyMethodDef module_methods[] = {
-	{"insert_phase",	py_bsdconv_insert_phase,	METH_VARARGS,
-		PyDoc_STR("insert_phase(conversion, codecs, phase_type, phasen) -> Insert conversion phase into bsdconv conversion string")},
-	{"insert_codec",	py_bsdconv_insert_codec,	METH_VARARGS,
-		PyDoc_STR("insert_codec(conversion, codec, phasen, codecn) -> Insert conversion codec into bsdconv conversion string")},
-	{"replace_phase",	py_bsdconv_replace_phase,	METH_VARARGS,
-		PyDoc_STR("replace_phase(conversion, codecs, phase_type, phasen) -> Replace conversion phase in the bsdconv conversion string")},
-	{"replace_codec",	py_bsdconv_replace_codec,	METH_VARARGS,
-		PyDoc_STR("replace_codec(conversion, codec, phasen, codecn) -> Replace conversion codec in the bsdconv conversion string")},
-	{"error",	py_bsdconv_error,	METH_VARARGS,
-		PyDoc_STR("error() -> Return error message")},
-	{"mktemp",	py_bsdconv_mktemp,	METH_VARARGS,
-		PyDoc_STR("mktemp() -> Make temporary file")},
-	{"fopen",	py_bsdconv_fopen,	METH_VARARGS,
-		PyDoc_STR("fopen() -> Open file")},
-	{"codecs_list",	py_bsdconv_codecs_list,	METH_VARARGS,
-		PyDoc_STR("codecs_list() -> list codecs")},
-	{"codec_check",	py_bsdconv_codec_check,	METH_VARARGS,
-		PyDoc_STR("codec_check(type, codec) -> check if a codec is available")},
 	{NULL,		NULL}		/* sentinel */
 };
 #if PY_MAJOR_VERSION >= 3
@@ -706,6 +710,17 @@ PyInit_bsdconv(void)
 {
 	PyObject *m;
 	Bsdconv_Type.tp_new = (newfunc)py_bsdconv_new;
+
+	Bsdconv_Type.tp_dict = PyDict_New();
+	PyDict_SetItemString(Bsdconv_Type.tp_dict, "FILTER", PyLong_FromLong(FILTER));
+	PyDict_SetItemString(Bsdconv_Type.tp_dict, "FROM", PyLong_FromLong(FROM));
+	PyDict_SetItemString(Bsdconv_Type.tp_dict, "INTER", PyLong_FromLong(INTER));
+	PyDict_SetItemString(Bsdconv_Type.tp_dict, "TO", PyLong_FromLong(TO));
+
+	PyDict_SetItemString(Bsdconv_Type.tp_dict, "CTL_ATTACH_SCORE", PyLong_FromLong(BSDCONV_CTL_ATTACH_SCORE));
+	PyDict_SetItemString(Bsdconv_Type.tp_dict, "CTL_ATTACH_OUTPUT_FILE", PyLong_FromLong(BSDCONV_CTL_ATTACH_OUTPUT_FILE));
+	PyDict_SetItemString(Bsdconv_Type.tp_dict, "CTL_AMBIGUOUS_PAD", PyLong_FromLong(BSDCONV_CTL_AMBIGUOUS_PAD));
+
 #if PY_MAJOR_VERSION < 3
 	if (PyType_Ready(&Bsdconv_Type) < 0)
 		return;
@@ -721,17 +736,10 @@ PyInit_bsdconv(void)
 		return NULL;
 	}
 #endif
+
 	Py_INCREF(&Bsdconv_Type);
 	PyModule_AddObject(m, "Bsdconv", (PyObject *)&Bsdconv_Type);
 
-	PyModule_AddIntConstant(m, "FROM", FROM);
-	PyModule_AddIntConstant(m, "INTER", INTER);
-	PyModule_AddIntConstant(m, "TO", TO);
-
-	PyModule_AddIntConstant(m, "CTL_ATTACH_SCORE", BSDCONV_ATTACH_SCORE);
-	PyModule_AddIntConstant(m, "CTL_SET_WIDE_AMBI", BSDCONV_SET_WIDE_AMBI);
-	PyModule_AddIntConstant(m, "CTL_SET_TRIM_WIDTH", BSDCONV_SET_TRIM_WIDTH);
-	PyModule_AddIntConstant(m, "CTL_ATTACH_OUTPUT_FILE", BSDCONV_ATTACH_OUTPUT_FILE);
 #if PY_MAJOR_VERSION >= 3
 	return m;
 #endif
